import 'dart:convert';

import 'package:equatable/equatable.dart';

/// Represents the infoBody (information body) of a [Ticket].
class InfoBody extends Equatable {
  InfoBody(this.serviceName, this.operation, this.transactionIdentifier,
      this.timestamp, this.validityPeriod, this.payload);

  /// The name of the service this [Ticket] is supposed to go to
  final String serviceName;

  /// The operation the service should perform
  final String operation;

  /// The identifier for the transaction this [Ticket] is starting
  final List<int> transactionIdentifier;

  /// The timestamp this [Ticket] was generated on
  final int timestamp;

  /// How long this [Ticket] is valid for
  final Duration validityPeriod;

  /// The payload - must be serializable
  final dynamic payload;

  /// Checks if this [InfoBody] is valid
  bool isValid() {
    // check if transactionIdentifier is at least 6 bytes
    if (transactionIdentifier.length < 6) return false;

    // check if the validityPeriod is non-negative
    if (validityPeriod.inMilliseconds < 0) return false;

    // check if the validity period has passed
    if (validityPeriod.inMilliseconds + timestamp <
        DateTime.now().millisecondsSinceEpoch) {
      return false;
    }

    return true;
  }

  /// Returns a [Map] representing this [InfoBody]
  Map toMap() {
    return {
      'serviceName': serviceName,
      'operation': operation,
      'transactionIdentifier': transactionIdentifier,
      'timestamp': timestamp,
      'validityPeriod': validityPeriod.inMilliseconds,
      'payload': payload
    };
  }

  /// Returns a JSON string representing this [InfoBody]
  String toJson() {
    return jsonEncode(toMap());
  }

  /// Returns an [InfoBody] instance, reconstructed from a given [Map]
  factory InfoBody.fromMap(Map map) {
    // check if the infobody is missing anything
    if (map['serviceName'] == null ||
        map['operation'] == null ||
        map['transactionIdentifier'] == null ||
        map['timestamp'] == null ||
        map['validityPeriod'] == null) {
      throw FormatException('Malformed infoBody');
    }

    // runtime type checking

    // check if serviceName and operation are Strings
    if (map['serviceName'] is! String || map['operation'] is! String) {
      throw FormatException(
          'Malformed infoBody - serviceName and/or operation are not of type String');
    }

    // check if transactionIdentifier is of type List<int>
    if (map['transactionIdentifier'] is! List) {
      throw FormatException(
          'Malformed infoBody - transactionIdentifier is not of type List');
    } else {
      try {
        List<int>.from(map['transactionIdentifier']);
      } catch (e) {
        throw FormatException(
            'Malformed infoBody - transactionIdentifier is not of type List<int>');
      }
    }

    // check if timestamp and validityPeriod are of type int
    if (map['timestamp'] is! int || map['validityPeriod'] is! int) {
      throw FormatException(
          'Malformed infoBody - timestamp and/or validityPeriod are not of type int');
    }

    return InfoBody(
        map['serviceName'],
        map['operation'],
        List<int>.from(map['transactionIdentifier']),
        map['timestamp'],
        Duration(milliseconds: map['validityPeriod']),
        map['payload']);
  }

  /// Returns an [InfoBody] instance, reconstructed from a given JSON string
  factory InfoBody.fromJson(String json) {
    return InfoBody.fromMap(jsonDecode(json));
  }

  @override
  List<Object> get props => [
        serviceName,
        operation,
        transactionIdentifier,
        timestamp,
        validityPeriod,
        payload
      ];
}
