import 'dart:convert';

import 'package:ticket/ticket.dart';
import 'package:test/test.dart';

void main() {
  group('InfoBody', () {
    InfoBody infoBody;

    setUp(() {
      infoBody = InfoBody('serviceName', 'operation', [0, 0, 0, 0, 0, 0],
          DateTime.now().millisecondsSinceEpoch, Duration(minutes: 1), 'h');
    });

    test('toMap()', () {
      expect(
          infoBody.toMap(),
          equals({
            'serviceName': infoBody.serviceName,
            'operation': infoBody.operation,
            'transactionIdentifier': infoBody.transactionIdentifier,
            'timestamp': infoBody.timestamp,
            'validityPeriod': infoBody.validityPeriod.inMilliseconds,
            'payload': infoBody.payload
          }));
    });

    group('fromMap()', () {
      test('No issues', () {
        expect(
            InfoBody.fromMap({
              'serviceName': infoBody.serviceName,
              'operation': infoBody.operation,
              'transactionIdentifier': infoBody.transactionIdentifier,
              'timestamp': infoBody.timestamp,
              'validityPeriod': infoBody.validityPeriod.inMilliseconds,
              'payload': infoBody.payload
            }),
            equals(infoBody));
      });

      test('Expect a null field to throw a FormatException', () {
        expect(() {
          InfoBody.fromMap({
            'serviceName': null,
            'operation': '',
            'transactionIdentifier': infoBody.transactionIdentifier,
            'timestamp': infoBody.timestamp,
            'validityPeriod': infoBody.validityPeriod.inMilliseconds,
            'payload': infoBody.payload
          });
        }, throwsA(isA<FormatException>()));
      });

      test(
          'Expect a non-String serviceName or operation to throw a FormatException',
          () {
        expect(() {
          InfoBody.fromMap({
            'serviceName': 0,
            'operation': 0,
            'transactionIdentifier': infoBody.transactionIdentifier,
            'timestamp': infoBody.timestamp,
            'validityPeriod': infoBody.validityPeriod.inMilliseconds,
            'payload': infoBody.payload
          });
        }, throwsA(isA<FormatException>()));
      });

      test(
          'Expect a non-int timestamp or validityPeriod to throw a FormatException',
          () {
        expect(() {
          InfoBody.fromMap({
            'serviceName': infoBody.serviceName,
            'operation': infoBody.operation,
            'transactionIdentifier': infoBody.transactionIdentifier,
            'timestamp': 'h',
            'validityPeriod': 'h',
            'payload': infoBody.payload
          });
        }, throwsA(isA<FormatException>()));
      });

      test(
          'Expect a non-List<int> transactionIdentifier to throw a FormatException',
          () {
        // not a List at all
        expect(() {
          InfoBody.fromMap({
            'serviceName': infoBody.serviceName,
            'operation': infoBody.operation,
            'transactionIdentifier': 'h',
            'timestamp': infoBody.timestamp,
            'validityPeriod': infoBody.validityPeriod.inMilliseconds,
            'payload': infoBody.payload
          });
        }, throwsA(isA<FormatException>()));

        // List but not a List<int>
        expect(() {
          InfoBody.fromMap({
            'serviceName': infoBody.serviceName,
            'operation': infoBody.operation,
            'transactionIdentifier': ['h'],
            'timestamp': infoBody.timestamp,
            'validityPeriod': infoBody.validityPeriod.inMilliseconds,
            'payload': infoBody.payload
          });
        }, throwsA(isA<FormatException>()));
      });
    });

    test('toJson()', () {
      expect(infoBody.toJson(), equals(jsonEncode(infoBody.toMap())));
    });

    test('fromJson()', () {
      expect(InfoBody.fromJson(infoBody.toJson()), equals(infoBody));
    });

    group('.isValid()', () {
      test('returns true for good input', () {
        expect(infoBody.isValid(), equals(true));
      });

      group('returns false', () {
        test('when the validity period has closed', () {
          expect(
              InfoBody('h', 'h', [0, 1, 2, 3, 4, 5], infoBody.timestamp - 30000,
                      Duration(microseconds: 0), 0)
                  .isValid(),
              equals(false));
        });

        test('when the transactionIdentifier is short', () {
          expect(
              InfoBody('h', 'h', [0], infoBody.timestamp,
                      infoBody.validityPeriod, 0)
                  .isValid(),
              equals(false));
        });

        test('when the validityPeriod is negative', () {
          expect(
              InfoBody('h', 'h', [0, 1, 2, 3, 4, 5], infoBody.timestamp,
                      Duration(milliseconds: -1), 0)
                  .isValid(),
              equals(false));
        });
      });
    });
  });
}
